package blueeyes.io.iteratee

import Input._
import Iteratee._

trait Enumerator[E] { self =>
  def apply[A](e: Iteratee[E, A]): Iteratee[E, A]
  
  def >>> (e2: Enumerator[E]): Enumerator[E] = new Enumerator[E] {
    def apply[A](iter: Iteratee[E, A]): Iteratee[E, A] = {
      e2.apply(self.apply(iter))
    }
  }
}

trait CoreEnumerators {

}
object CoreEnumerators extends CoreEnumerators

trait ChunkEnumerators extends CoreEnumerators {
  def repeat[E](e: E, n: Int) = new Enumerator[E] {
    private def repeat0[A](e: E, count: Int, iter: Iteratee[E, A]): Iteratee[E, A] = {
      if (n <= 0) iter
      else iter match {
        case Done(a, r)   => Iteratee.returnValue(a, r)
        case Next(n)      => repeat0(e, count - 1, n(Chunk(e)))
        case Error(e, n)  => Iteratee.throwErr(e, n)
      }
    }
    
    def apply[A](iter: Iteratee[E, A]) = repeat0(e, n, iter)
  }
  
  def fromIterable[E](l: Iterable[E]): Enumerator[E] = new Enumerator[E] {
    def apply[A](iter: Iteratee[E, A]) = {
      l.headOption match {
        case None => iter
        
        case Some(head) => iter match {
          case Done(a, r)   => Iteratee.returnValue(a, r)
          case Next(n)      => fromIterable(l.tail).apply(n(Chunk(head)))
          case Error(e, n)  => Iteratee.throwErr(e, n)
        }
      }
    }
  }
}
object ChunkEnumerators extends ChunkEnumerators

object Enumerator extends CoreEnumerators {

}
