package blueeyes.io.iteratee

/** A single unit of input, which can contain nothing, or a chunk of data, or 
 * an end-of-file marker.
 */
sealed trait Input[+E] {
  def map[EE](f: E => EE): Input[EE]
  
  def flatMap[EE](f: E => Input[EE]): Input[EE]
  
  def filter(f: E => Boolean): Input[E]
}

object Input {
  case object Empty extends Input[Nothing] {
    def map[EE](f: Nothing => EE): Input[EE] = this

    def flatMap[EE](f: Nothing => Input[EE]): Input[EE] = this

    def filter(f: Nothing => Boolean): Input[Nothing] = this
  }
  
  case object Eof extends Input[Nothing] {
    def map[EE](f: Nothing => EE): Input[EE] = this

    def flatMap[EE](f: Nothing => Input[EE]): Input[EE] = this

    def filter(f: Nothing => Boolean): Input[Nothing] = this
  }
  
  sealed case class Chunk[E](e: E) extends Input[E] {
    def map[EE](f: E => EE): Input[EE] = Chunk(f(e))

    def flatMap[EE](f: E => Input[EE]): Input[EE] = f(e)

    def filter(f: E => Boolean): Input[E] = if (f(e)) this else Empty
  }
}